@@ -40,6 +40,10 @@
 
 /* Added header for getnameinfo() */
 #include <netdb.h>
+#include "sync_session.h" 
+
+static void shm_sem_remove(); 
+static void stop_l7sync_daemons();
 
 static int l7vs_config_callback(struct l7vs_iomux *iom, int flags);
 static void l7vs_config_dispatch(union l7vs_config_req *req,
@@ -66,6 +70,7 @@ static int l7vs_config_send_response(struct sockaddr_un *addr,
                                      struct iovec *iov,
                                      int iovlen);
 
+
 /* For message queue / Trap */
 int check_process_l7vsagent(char *);
 static char * get_servicename(struct sockaddr_in *, int);
@@ -147,6 +152,8 @@ l7vs_config_fini(void)
         l7vs_iomux_remove(&l7vs_config_iomux);
         close(l7vs_config_iomux.fd);
         unlink(l7vs_config_sockname);
+	stop_l7sync_daemons();
+	shm_sem_remove();
 }
 
 static int
@@ -378,7 +385,6 @@ l7vs_config_add_vs(union l7vs_config_req *arg,
                 rsp.code = L7VS_CONFIG_ERR_VS_EXISTS;
                 goto out;
         }
-
         /* Keep the message ready to be sent */
         srvname = get_servicename(&sarg->addr, 1);
         sprintf(trap_message, "Add Virtual Service:%s module %s scheduler %s", srvname, sarg->protomod, sarg->schedmod);
@@ -652,6 +658,7 @@ l7vs_config_flush_vs(union l7vs_config_req *arg, struct sockaddr_un *addr)
 
         l7vs_service_flush_all();
 
+
         /* Send signal SIGUSR1 to l7vsagent process */
         check_process_l7vsagent("Clear L7VS Table");
 
@@ -697,6 +704,7 @@ int check_process_l7vsagent(char *trap_message)
 
         /* message related */
         char trap_string[BUFSIZ];
+
         key_t key;
         int msgid;
 
@@ -735,8 +743,8 @@ int check_process_l7vsagent(char *trap_message)
                         return 0;
 
                 /* Send the message to the queue */
-                strcpy(trap_string, trap_message);
-                if (msgsnd(msgid, (void *) &trap_string, sizeof(trap_string), 0) == -1)
+                strcpy(trap_string, trap_message); 
+                if (msgsnd(msgid, (void *) &trap_string, sizeof(trap_string), 0) == -1) 
                         return 0;
 
                 /* Sending SIGUSR1 to l7vsagent */
@@ -781,3 +789,42 @@ static char * get_servicename(struct sockaddr_in *addr, int isnumeric)
 
         return sname;
 }
+
+static void stop_l7sync_daemons()
+{
+	FILE *fp;
+	fp=fopen("/etc/init.d/l7syncd","r");
+        if (fp != NULL)
+		system("/etc/init.d/l7syncd stopall");
+        fclose(fp);
+}
+
+void shared_mem_sem_remove(key_t shmid, key_t sema_id)
+{
+        if (semctl(sema_id, IPC_RMID, 0) == -1)
+        {
+                fprintf(stderr, "semctl(IPC_RMID) failed reason %s\n",strerror(errno));
+                fprintf(stderr, "semctl(IPC_RMID) failed\n");
+        }
+
+        if (shmctl(shmid, IPC_RMID, 0) == -1)
+        {
+                fprintf(stderr, "shmctl(IPC_RMID) failed reason %s\n",strerror(errno));
+                fprintf(stderr, "shmctl(IPC_RMID) failed\n");
+        }
+}
+
+static void shm_sem_remove() 
+{
+        int ck_semid,ck_shmid;
+	int evs_semid,evs_shmid;
+
+        ck_semid=initialize_sem(key_sem_ck);
+        ck_shmid=initialize_shm(key_shm_ck,sizeof(struct sync_info_st));
+	shared_mem_sem_remove(ck_shmid, ck_semid);
+
+        evs_semid=initialize_sem(key_sem_evs);
+	evs_shmid=initialize_shm(key_shm_evs,sizeof(struct evasion));
+	shared_mem_sem_remove(evs_shmid, evs_semid);
+}
+
