@@ -82,7 +82,7 @@ l7vs_conn_create(int lfd, struct l7vs_lsock *lsock)
         if (ret < 0) {
                 VANESSA_LOGGER_ERR_UNSAFE("getsockopt TCP_MAXSEG: %s",
                                           strerror(errno));
-		close(conn->ciom.fd);
+                close(conn->ciom.fd);
                 free(conn->cldata);
                 free(conn);
                 return NULL;
@@ -92,20 +92,10 @@ l7vs_conn_create(int lfd, struct l7vs_lsock *lsock)
                                     inet_ntoa(conn->caddr.sin_addr), mss);
 
         len = sizeof(conn->tcpopt);
-        ret = getsockopt(conn->ciom.fd, IPPROTO_IP, TCPS_SO_GET_TCPOPT,
-                         &conn->tcpopt, &len);
-        if (ret < 0) {
-                VANESSA_LOGGER_ERR_UNSAFE("getsockopt TCPS_SO_GET_TCPOPT: %s",
-                                          strerror(errno));
-		close(conn->ciom.fd);
-                free(conn->cldata);
-                free(conn);
-                return NULL;
-        }
 
         conn->lsock = lsock;
         conn->proto = lsock->proto;
-        conn->state = L7VS_CONN_S_CL_CONNECTED;
+        conn->state = L7VS_CONN_S_CL_CONNECTED; 
         conn->ciom.callback = l7vs_conn_cl_callback;
         conn->ciom.data = conn;
         l7vs_iomux_add(&conn->ciom, L7VS_IOMUX_READ);
@@ -177,7 +167,7 @@ l7vs_conn_cl_callback(struct l7vs_iomux *iom, int flags)
                         break;
                 }
 
-                conn->splice = tcps;
+		conn->splice = 0;
                 ret = l7vs_conn_connect_rs(conn, dest);
                 if (ret < 0) {
                         goto cl_recv_err;
@@ -201,15 +191,23 @@ cl_recv_err:
                 return L7VS_IOMUX_LIST_REMOVED_OTHER;
 
         case L7VS_CONN_S_RS_CONNECTING:
-                l7vs_conn_destroy(conn);
-                return L7VS_IOMUX_LIST_REMOVED_OTHER;
+
+		ret = l7vs_conn_recv_client(conn);
+		if (ret <= 0) {
+			l7vs_conn_destroy(conn);
+			return L7VS_IOMUX_LIST_REMOVED_OTHER;
+		} else {
+			return L7VS_IOMUX_LIST_UNCHANGED;
+		}
 
         case L7VS_CONN_S_RS_CONNECTED:
                 ret = l7vs_conn_relay_cl(conn);
                 if (ret <= 0) {
+			int splice = conn->splice;
+
                         /* Client connection closed. */
                         l7vs_conn_close_csock(conn);
-                        if (!conn->splice) {
+                        if (!splice) {
                                 l7vs_conn_close_rsock(conn);
                         }
 
@@ -217,7 +215,7 @@ cl_recv_err:
                                 l7vs_conn_destroy(conn);
                         }
 
-                        if (conn->splice) {
+                        if (splice) {
                                 return L7VS_IOMUX_LIST_REMOVED_MYSELF;
                         } else {
                                 return L7VS_IOMUX_LIST_REMOVED_OTHER;
@@ -237,6 +235,7 @@ l7vs_conn_rs_callback(struct l7vs_iomux *iom, int flags)
         socklen_t len;
         int opt;
         int ret;
+	int fcntl_flags; 
 
         conn = (struct l7vs_conn *)iom->data;
         lsock = conn->lsock;
@@ -284,15 +283,28 @@ l7vs_conn_rs_callback(struct l7vs_iomux *iom, int flags)
                         l7vs_conn_destroy(conn);
                         return L7VS_IOMUX_LIST_REMOVED_OTHER;
                 }
-
-                break;
+		fcntl_flags = fcntl(conn->riom.fd, F_GETFL, 0);
+		if (fcntl_flags < 0) {
+			VANESSA_LOGGER_ERR_UNSAFE("fcntl(F_GETFL): %s", strerror(errno));
+			l7vs_conn_destroy(conn);
+			return L7VS_IOMUX_LIST_REMOVED_OTHER;
+		}
+		fcntl_flags = fcntl(conn->riom.fd, F_SETFL, fcntl_flags & ~O_NONBLOCK);
+		if (fcntl_flags < 0) {
+			VANESSA_LOGGER_ERR_UNSAFE("fcntl(F_SETFL): %s", strerror(errno));
+			l7vs_conn_destroy(conn);
+			return L7VS_IOMUX_LIST_REMOVED_OTHER;
+		}
+		break;
 
         case L7VS_CONN_S_RS_CONNECTED:
                 ret = l7vs_conn_relay_rs(conn);
                 if (ret <= 0) {
+			int splice = conn->splice;
+
                         /* Connection closed. */
                         l7vs_conn_close_rsock(conn);
-                        if (!conn->splice) {
+                        if (!splice) {
                                 l7vs_conn_close_csock(conn);
                         }
 
@@ -300,7 +312,7 @@ l7vs_conn_rs_callback(struct l7vs_iomux *iom, int flags)
                                 l7vs_conn_destroy(conn);
                         }
 
-                        if (conn->splice) {
+                        if (splice) {
                                 return L7VS_IOMUX_LIST_REMOVED_MYSELF;
                         } else {
                                 return L7VS_IOMUX_LIST_REMOVED_OTHER;
@@ -330,7 +342,7 @@ l7vs_conn_rs_connected(struct l7vs_conn *conn)
                 return ret;
         }
         
-        conn->state = L7VS_CONN_S_RS_CONNECTED;
+        conn->state = L7VS_CONN_S_RS_CONNECTED; 
         conn->dest->nactive++;
 
         return send(conn->riom.fd, conn->cldata, conn->cldata_len, 0);
@@ -419,10 +431,8 @@ l7vs_conn_relay(struct l7vs_conn *conn, int is_client)
                 }
         }
 
-        len = recv(rfd, buf, sizeof(buf), 0);
+        len = recv(rfd, buf,L7VS_CONN_RELAY_BUFSIZE , 0);
         if (len < 0) {
-//                VANESSA_LOGGER_ERR_UNSAFE("recv on %s fd: %s\n",
-//                                          rcvr, strerror(errno));
                 return len;
         } else if (len == 0) {
                 VANESSA_LOGGER_DEBUG_UNSAFE("%s connection closed\n", rcvr);
@@ -497,7 +507,7 @@ l7vs_conn_connect_rs(struct l7vs_conn *conn, struct l7vs_dest *dest)
                 return -1;
         }
 
-        s = socket(PF_INET, stype, conn->proto);
+        s = socket(PF_INET, stype, conn->proto);  
         if (s < 0) {
                 VANESSA_LOGGER_ERR_UNSAFE("socket: %s", strerror(errno));
                 return s;
@@ -519,16 +529,6 @@ l7vs_conn_connect_rs(struct l7vs_conn *conn, struct l7vs_dest *dest)
                 return flags;
         }
 
-        ret = setsockopt(s, IPPROTO_IP, TCPS_SO_SET_SOURCE_FAKE, &conn->caddr,
-                         sizeof(conn->caddr));
-        if (ret < 0) {
-                VANESSA_LOGGER_ERR_UNSAFE(
-                                "setsockopt(TCPS_SO_SET_SOURCE_FAKE: %s",
-                                strerror(errno));
-                close(s);
-                return ret;
-        }
-
         if (conn->cmss > 0) {
                 ret = setsockopt(s, IPPROTO_TCP, TCP_MAXSEG, &conn->cmss,
                                  sizeof(conn->cmss));
@@ -539,25 +539,15 @@ l7vs_conn_connect_rs(struct l7vs_conn *conn, struct l7vs_dest *dest)
                 }
         }
 
-        ret = setsockopt(s, IPPROTO_IP, TCPS_SO_SET_TCPOPT, &conn->tcpopt,
-                         sizeof(conn->tcpopt));
-        if (ret < 0) {
-                VANESSA_LOGGER_ERR_UNSAFE(
-                                "setsockopt(TCPS_SO_SET_TCPOPT): %s",
-                                strerror(errno));
-                close(s);
-                return ret;
-        }
-
         ret = connect(s, (struct sockaddr *)&dest->addr,
-                      sizeof(dest->addr));
+                      sizeof(dest->addr)); 
         if (ret >= 0) {
-                conn->state = L7VS_CONN_S_RS_CONNECTED;
+                conn->state = L7VS_CONN_S_RS_CONNECTED; 
                 VANESSA_LOGGER_DEBUG_UNSAFE("dest %p nactive %d->%d (imm.)",
                                             conn->dest, conn->dest->nactive,
                                             conn->dest->nactive + 1);
                 iomux_flags = L7VS_IOMUX_READ;
-                conn->dest->nactive++;
+                conn->dest->nactive++;   
         } else {
                 if (errno != EINPROGRESS) {
                         VANESSA_LOGGER_ERR_UNSAFE("connect: %s", 
@@ -568,7 +558,7 @@ l7vs_conn_connect_rs(struct l7vs_conn *conn, struct l7vs_dest *dest)
                 conn->state = L7VS_CONN_S_RS_CONNECTING;
                 iomux_flags = L7VS_IOMUX_WRITE;
         }
-        conn->dest = dest;
+        conn->dest = dest;   
 
         conn->riom.fd = s;
         conn->riom.callback = l7vs_conn_rs_callback;
