@@ -40,6 +40,7 @@
 #include <vanessa_adt.h>
 #include "glib.h"
 #include "l7vs.h"
+#include "sync_session.h"
 
 #define L7VSADM_NAME            "l7vsadm"
 #define L7VSADM_LIST_VS_BUFSIZE 1024
@@ -94,6 +95,7 @@ static int recv_response(int s, struct iovec *iov, size_t iovlen);
 static const char *config_strerror(int err);
 static int list_vs(int s, struct l7vsadm_options *opt);
 static int list_rs(int s, struct l7vs_service_arg *sarg, int isnumeric);
+void list_ss(struct sockaddr_in);
 static int operate_vs(int s, struct l7vsadm_options *opt);
 static int operate_rs(int s, struct l7vsadm_options *opt);
 static void sig_exit_handler(int sig);
@@ -103,12 +105,23 @@ static const char l7vs_config_sockname[] = L7VS_CONFIG_SOCK_PATH "/l7vs";
 static const char *l7vs_version_string = L7VS_VERSION_STRING;
 static char *local_sockname;
 
+char vaddr[16];
+char ssaddr[16];
+struct sockaddr_in ss_addr;
+struct sockaddr_in v_addr;
+int cc;
+struct evasion *evs;
+int shmid;
+void *shared_memory=(void *)0;
+int e_cc_flag = 0;
+int e_ss_flag = 0;
+
 static void
 usage(FILE *fp)
 {
         fprintf(fp,
                 "Usage: \n"
-                "  l7vsadm -A|E -t service-address -m proto-module [module-args] [-s scheduler]\n"
+                "  l7vsadm -A|E -t service-address -m proto-module [module-args] [-s scheduler] [-u connecttion-count] [-b sorry-server]\n"
                 "  l7vsadm -D -t service-address -m proto-module [module-args]\n"
                 "  l7vsadm -C\n"
 #if 0
@@ -136,7 +149,10 @@ usage(FILE *fp)
                 "  --edit-server     -e        edit real server with options\n"
                 "  --list            -L|l      list the table\n\n"
                 "  --verbose         -V        list the table in verbose format\n"
+		"  --upper	     -u        maximum number of connections\n"
+		"  --bypass          -b       sorry server ip address\n"
                 "  --key             -K        list the table in key setting format\n\n"
+		
                 );
         fprintf(fp,
                 "Options:\n"
@@ -150,7 +166,9 @@ usage(FILE *fp)
 static int
 parse_args(struct l7vsadm_options *opt, int argc, char *argv[])
 {
-#define OPTION_STRING   "-lVKADECadenr:m:t:s:w:"
+	int semid; 
+
+#define OPTION_STRING   "-lVKADECadenr:m:t:s:w:u:b:"
 
         static struct option options[] = {
                 /* operation options (toplevel) */
@@ -172,6 +190,8 @@ parse_args(struct l7vsadm_options *opt, int argc, char *argv[])
                 {"scheduler",           required_argument,      NULL, 's'},
 
                 {"weight",              required_argument,      NULL, 'w'},
+		{"connection-count",	required_argument,	NULL, 'u'}, 
+		{"sorry-server",	required_argument,	NULL, 'b'}, 
 
                 {"numeric",             no_argument,            NULL, 'n'},
 
@@ -182,6 +202,7 @@ parse_args(struct l7vsadm_options *opt, int argc, char *argv[])
 
         int c;
         int ret;
+	int edit_flag=0,add_flag=0;
 
 #define SET_OP(OPTP, OP) \
 do {                                                                    \
@@ -207,12 +228,14 @@ do {                                                                    \
 			break;
                 case 'A':
                         SET_OP(opt, L7VSADM_OP_ADD_VS);
+			add_flag=1;
                         break;
                 case 'D':
                         SET_OP(opt, L7VSADM_OP_DEL_VS);
                         break;
                 case 'E':
                         SET_OP(opt, L7VSADM_OP_EDIT_VS);
+			edit_flag=1;
                         break;
                 case 'C':
                         SET_OP(opt, L7VSADM_OP_FLUSH_VS);
@@ -234,6 +257,8 @@ do {                                                                    \
                                                 optarg);
                                 exit(1);
                         }
+			v_addr = opt->vaddr;
+			strcpy(vaddr,inet_ntoa(opt->vaddr.sin_addr));
                         break;
                 case 'r':
                         ret = parse_service(&opt->raddr, optarg);
@@ -295,6 +320,28 @@ do {                                                                    \
                 case 'n':
                         opt->isnumeric = 1;
                         break;
+		case 'u':
+			cc = atoi(optarg);
+			if(cc<=0)
+			{
+			  VANESSA_LOGGER_ERR_UNSAFE("Should not have -ve values %d", cc);
+			  exit(1);
+			}
+			e_cc_flag=1;
+			break;
+		case 'b':
+                        ret = parse_service(&ss_addr, optarg);
+                        if (ret < 0) {
+                                VANESSA_LOGGER_ERR_UNSAFE("Could not parse"
+                                                " service string: %s",
+                                                optarg);
+                                exit(1);
+                        }
+			
+			strcpy(ssaddr,inet_ntoa(ss_addr.sin_addr));
+			e_ss_flag=1;
+                        break;
+ 
 
                 case 'h':
                         usage(stdout);
@@ -312,8 +359,62 @@ do {                                                                    \
                         }
                         break;
                 }
-
         }
+	//Begin : writting evasion parameters into shared memory
+		if((e_cc_flag==1)||(e_ss_flag==1))
+		{
+			semid=initialize_sem(key_sem_evs);
+                        shmid = initialize_shm(key_shm_evs,sizeof(struct evasion));
+                        shared_memory = shmat(shmid,0,0);
+		        if (shared_memory == (void *)-1)
+		        {
+                		VANESSA_LOGGER_ERR("l7vsadm evasion shmat failed");
+		        }
+
+                        evs = (struct evasion *)shared_memory;
+                        int i;
+			semaphore_lock(semid);
+                        for (i=0;i<LE_VIP_SZ;i++)
+                        {
+			   if (strcmp(vaddr,evs->e[i].vaddr)==0)
+                        	{
+				 if((edit_flag==1) && (e_cc_flag==1))
+                   		        evs->e[i].cc = cc;
+				 if((edit_flag==1) && (e_ss_flag==1))
+                                        strcpy(evs->e[i].ssaddr,ssaddr);
+                                 break;
+                        	}
+                        else if(strcmp(evs->e[i].vaddr,"")==0)
+                            {
+				if((add_flag==1)||(edit_flag==1))
+				{
+					if((e_cc_flag==1)&&(e_ss_flag==1))
+					{
+                                  	strcpy(evs->e[i].vaddr,vaddr);
+				  	strcpy(evs->e[i].ssaddr,ssaddr);
+                                  	evs->e[i].cc = cc;
+					}
+					else if(e_ss_flag==0)
+					{
+						VANESSA_LOGGER_ERR("Warning -b option is missing ");
+					}
+					else if (e_cc_flag==0)
+					{
+						VANESSA_LOGGER_ERR("Warning -u option is missing ");
+					}
+				  break;
+				}
+				break;
+                            }
+
+                        }
+			semaphore_unlock(semid);
+		        if (shmdt(shared_memory) == -1)
+        		{
+		                VANESSA_LOGGER_ERR("l7vsadm evasion shmdt failed");
+		        }
+		}
+	//End : writting evasion parameters into shared memory
 #undef SET_OP
 
         return 0;
@@ -602,6 +703,7 @@ list_vs(int s, struct l7vsadm_options *opt)
         int bufsize;
         int i, off;
         int ret;
+	int verbose_flag=0;
 
         bufsize = L7VSADM_LIST_VS_BUFSIZE;
 
@@ -658,6 +760,7 @@ retry:
 
 		/* If it ie verbose, display more details else simple details */
 		if (opt->operation == L7VSADM_OP_VERBOSE) {
+			verbose_flag = 1;
                 	printf("TCP %s %s %s %d %s\n",
                        srvname, sarg->protomod, sarg->schedmod, sarg->reschedule, sarg->protomod_opt_string);
 		} else if (opt->operation == L7VSADM_OP_KEY) {
@@ -669,8 +772,12 @@ retry:
 		}
 
                 free(srvname);
-                list_rs(s, sarg, opt->isnumeric);
+		list_rs(s, sarg, opt->isnumeric);
 
+	//Begin : print sorry server details
+		if(verbose_flag == 1)
+			list_ss(sarg->addr);
+	//End   : print sorry server details 
                 off += sarg->len;
                 if (off >= bufsize) {
                         VANESSA_LOGGER_INFO("Message size did not match");
@@ -680,7 +787,6 @@ retry:
                 }
                 sarg = (struct l7vs_service_arg *)((char *)buf + off);
         }
-
         free(buf);
 
         return 0;
@@ -1118,3 +1224,32 @@ main(int argc, char *argv[])
 
         return 0;
 }
+
+/* Displays sorry server and maximum connection count for each VIP */
+void list_ss(struct sockaddr_in vaddr)
+{
+	char *srv,*cp;
+	const char token[] = ":";
+	int shmid;
+	struct evasion *evs;
+
+	shmid = initialize_shm(key_shm_evs, sizeof(struct evasion));
+        shared_memory = shmat(shmid,0,0);
+        if (shared_memory == (void *)-1)
+        {
+       		VANESSA_LOGGER_ERR("l7vsadm evasion shmat failed");
+        }
+
+        evs = (struct evasion *)shared_memory;
+        int i;
+	
+	for (i=0;i<LE_VIP_SZ;i++)
+	{
+		if(strcmp(evs->e[i].vaddr,inet_ntoa(vaddr.sin_addr))==0)
+		{
+		  printf("sorryserver : %s\tmaxconn : %d\n",evs->e[i].ssaddr,evs->e[i].cc);
+		  break;
+		}
+	}
+}
+
